@@ -581,6 +581,28 @@ class DeltaTable(object):
             getattr(self._spark, "_wrapped", self._spark)  # type: ignore[attr-defined]
         )
 
+    @since(2.0)  # type: ignore[arg-type]
+    def optimize(self) -> "DeltaOptimizeBuilder":
+        """
+        Optimize the data layout of the table. This returns
+        a :py:class:`~delta.tables.DeltaOptimizeBuilder` object that can
+        be used to specify the partition filter to limit the scope of
+        optimize and also execute different optimization techniques
+        such as file compaction or order data using Z-Order curves.
+
+        See the :py:class:`~delta.tables.DeltaOptimizeBuilder` for a
+        full description of this operation.
+
+        Example::
+
+            deltaTable.optimize().where("date='2021-11-18'").executeCompaction()
+
+        :return: an instance of DeltaOptimizeBuilder.
+        :rtype: :py:class:`~delta.tables.DeltaOptimizeBuilder`
+        """
+        jbuilder = self._jdt.optimize()
+        return DeltaOptimizeBuilder(self._spark, jbuilder)
+
     @staticmethod  # type: ignore[arg-type]
     def _verify_type_str(variable: str, name: str) -> None:
         if not isinstance(variable, str) or variable is None:
@@ -647,9 +669,8 @@ class DeltaMergeBuilder(object):
     """
     Builder to specify how to merge data from source DataFrame into the target Delta table.
     Use :py:meth:`delta.tables.DeltaTable.merge` to create an object of this class.
-    Using this builder, you can specify 1, 2 or 3 ``when`` clauses of which there can be at most
-    2 ``whenMatched`` clauses and at most 1 ``whenNotMatched`` clause.
-    Here are the constraints on these clauses.
+    Using this builder, you can specify any number of ``whenMatched`` and ``whenNotMatched``
+    clauses. Here are the constraints on these clauses.
 
     - Constraints in the ``whenMatched`` clauses:
 
@@ -1145,3 +1166,70 @@ class DeltaTableBuilder(object):
         """
         jdt = self._jbuilder.execute()
         return DeltaTable(self._spark, jdt)
+
+
+class DeltaOptimizeBuilder(object):
+    """
+    Builder class for constructing OPTIMIZE command and executing.
+
+    Use :py:meth:`delta.tables.DeltaTable.optimize` to create an instance of this class.
+
+    .. versionadded:: 2.0.0
+    """
+    def __init__(self, spark: SparkSession, jbuilder: "JavaObject"):
+        self._spark = spark
+        self._jbuilder = jbuilder
+
+    @since(2.0)  # type: ignore[arg-type]
+    def where(self, partitionFilter: str) -> "DeltaOptimizeBuilder":
+        """
+        Apply partition filter on this optimize command builder to limit
+        the operation on selected partitions.
+
+        :param partitionFilter: The partition filter to apply
+        :type partitionFilter: str
+        :return: DeltaOptimizeBuilder with partition filter applied
+        :rtype: :py:class:`~delta.tables.DeltaOptimizeBuilder`
+        """
+        self._jbuilder = self._jbuilder.where(partitionFilter)
+        return self
+
+    @since(2.0)  # type: ignore[arg-type]
+    def executeCompaction(self) -> DataFrame:
+        """
+        Compact the small files in selected partitions.
+
+        :return: DataFrame containing the OPTIMIZE execution metrics
+        :rtype: pyspark.sql.DataFrame
+        """
+        return DataFrame(
+            self._jbuilder.executeCompaction(),
+            getattr(self._spark, "_wrapped", self._spark)  # type: ignore[attr-defined]
+        )
+
+    @since(2.0)  # type: ignore[arg-type]
+    def executeZOrderBy(self, *cols: Union[str, List[str], Tuple[str, ...]]) -> DataFrame:
+        """
+        Z-Order the data in selected partitions using the given columns.
+
+        :param cols: the Z-Order cols
+        :type cols: str or list name of columns
+
+        :return: DataFrame containing the OPTIMIZE execution metrics
+        :rtype: pyspark.sql.DataFrame
+        """
+        if len(cols) == 1 and isinstance(cols[0], (list, tuple)):
+            cols = cols[0]  # type: ignore[assignment]
+        for c in cols:
+            if type(c) is not str:
+                errorMsg = "Z-order column must be str. "
+                errorMsg += "Found %s with type %s" % ((str(c)), str(type(c)))
+                raise TypeError(errorMsg)
+
+        return DataFrame(
+            self._jbuilder.executeZOrderBy(_to_seq(
+                self._spark._sc,  # type: ignore[attr-defined]
+                cast(Iterable[Union[Column, str]], cols)
+            )),
+            getattr(self._spark, "_wrapped", self._spark)  # type: ignore[attr-defined]
+        )
